# Observables

Observables are the glue that hold the library together. They often are invisible but interact in very powerful ways that you get more familiar with Solid they unlock a lot of potential. So consider this an intermediate to advanced topic. In the end there are only 3 objects we are talking about here in Solid: Signals, Syncs, and Selectors. And in most cases you will need to write them explicitly even if they are what are being used under the hood when you are using state.select or binding.

### Observable

This is Observable as defined by the [TC39 Proposal](https://github.com/tc39/proposal-observable). These are a standard way of representing streams, and follow a few key conventions. Mostly that they are cold, unicast, and push-based by default. What this means is that they do not do anything until subscribed to at which point they create the source, and do so for each subscription. So if you had an Observable from a DOM Event, subscribing would add an event listener for each function you pass. In so being unicast they aren't managing a list of subscribers. Finally being push you don't ask for the latest value, they tell you.

However, this behavior from a UI perspective is a bit undesireable, sort of unexpected, and gets in the way of the function of this library. In Solid.js we like our Observables to be hot and multicast. Luckily its easy conversion that we do at all points we accept Observables.  We do a conversion anyway to expose our Observables to pull-based interfaces, like State and computations so their dependencies can be automatically tracked. These pullable Observables are called Signals.

### Computation

A computation is calculation over a function execution that automatically dynamically tracks it's dependencies. In Solid there are 2 objects that are considered computations, Syncs and Selectors. While serving different purposes they both go through a cycle on execution where they release the previous executions dependencies, then execute grabbing the current dependencies. This constant subscribe and unsubscribe is largely why Cold Observables are no good, and in fact Solid doesn't release the subscription immediately to prevent any unnecessary temporary ripple effects from unsubscribing.

## Sync

This the simplest computation and technically isn't an Observable. It has no ability to be subscribed to, and doesn't notify change. Instead it just runs to keep things in sync when ever changes occur. This is the core piece used in rendering where side effects are necessary (ie.. rendering or updating the DOM) and have a low overhead of execution. They run immediately when created, and every time there after as dependencies change. They are also useful for debugging state changes.

    import { Sync, State } from 'solid-js';

    state = new State({count: 1});

    new Sync(() => {
      console.log(state.count);
    });
    state.set({count: state.count + 1});

    // 1
    // 2

By default Sync execution is defered to the next Microtask to reduce unnecessary recalculation but this can be controlled by the second argument options.

    new Sync(() => ...., {defer: false})

### Signals

These are the most primitive of the Observables in Solid, and resemble BehaviorSubjects in RxJS. They are multicast, and have no provider of data. Instead the next method is called on them manually to push updates to subscribers. Their value can be fetched from their value property useful for computations. The can manually be subscribed to as any other Observable.

    import { S } from 'solid-js'

    name$ = S('John');
    new Sync(() => {
      console.log(name$.value);
    });

    name$.next('Jane');

    // John
    // Jane

Its important to note this factory treats all non-Observables, non-Promises as basaic values. An array is just an array. Combine with other libraries if you need to use more complicated streams.

## Selector

These computations are also Observables. They are pure functions that allow the merging of mapping of several observable values. They are similar to combineLatest in RxJS except the dependencies aren't explicit and they also behave like flatMap. They support asynchronous return types allowing them to be the only method used in most places and are key to the state.select method.

    import { S, State } from 'solid-js'

    state = new State({count: 1});

    countx3$ = S(() => state.count * 3);
    new Sync(() => {
      console.log(countx3$.value);
    });

    state.set({count: state.count + 1});

    // 3
    // 6

### Stream

These are wrappers of existing of Observables and Promises. They trigger on changes to the underlying object and have all the characteristics of Signals.

    import { S } from 'solid-js'

    delaySec = new Promise((resolve) => {
      setTimeout(resolve, 1000)
    }).then(() => 5);

    delayedVal$ = S(delaySec)
    new Sync(() => {
      console.log(delayedVal$.value);
    });

    // undefined
    // 5                   after 1000ms

